/*
   ; Test the K213 Module
   ; ====================

   ;------------------------------------------------------------------------
   ; Author:    Franzi Edo.     The 03/04/98
   ; Modifs:
   ;
   ; Project:   KHEPERA
   ; Goal:              Test of the K213 driver.
   ;
   ; This module is (c) of Edoardo Franzi 1992-1998
   ; franzi@k-team.com
   ;
   ;------------------------------------------------------------------------
 */

#include	<sys/kos.h>
#include	<sys/k213.h>
#include 	<stdlib.h>
#include 	<stdio.h>

uint32 vIDProcess[2];

/* Function 1 */
/* >>>>>>>>>> */

void
function_1 ()
{
  int32 status;
  uint8 vVersion, vRevision;

  status = k213_getVersion ();
  if (status < 0)
    {
      printf ("Network Communication Problem\r\n");
    }
  else
    {
      vVersion = (uint8) (status >> 8);
      vRevision = (uint8) (status);
      printf ("Version %d.%d\r\n", vVersion, vRevision);
    }
}

/* Function 2 */
/* >>>>>>>>>> */

void
function_2 ()
{
  int32 status;
  uint8 *image;
  uint32 i;

  image = (uint8 *) malloc (64 * sizeof (uint8));
  if (image == 0)
    {
      printf ("Memory problem\r\n");
      return;
    }

  status = k213_get8BitImage (image);
  if (status < 0)
    {
      printf ("Network Communication Problem\r\n");
    }
  else
    {
      printf ("64 pixel image of 8 bits ... ");
      for (i = 0; i < 64; i++)
	printf ("%d ", image[i]);
      printf ("\r\n\r\n\r\n\r\n");
    }
  free (image);
}

/* Function 3 */
/* >>>>>>>>>> */

void
function_3 ()
{
  int32 status;
  uint8 *image;
  uint32 i;

  image = (uint8 *) malloc (32 * sizeof (uint8));
  if (image == 0)
    {
      printf ("Memory problem\r\n");
      return;
    }

  status = k213_get4BitImage (image);
  if (status < 0)
    {
      printf ("Network Communication Problem\r\n");
    }
  else
    {
      printf ("64 pixel image of 4 bits ... ");
      for (i = 0; i < 32; i++)
	printf ("%d %d", (image[i] >> 4), (image[i] & 0xF));
      printf ("\r\n\r\n\r\n\r\n");
    }
  free (image);
}

/* Function 4 */
/* >>>>>>>>>> */

void
function_4 ()
{
  int32 status;
  uint8 *image;
  uint32 i, j;

  image = (uint8 *) malloc (8 * sizeof (uint8));
  if (image == 0)
    {
      printf ("Memory problem\r\n");
      return;
    }

  for (j = 0; j < 64; j++)
    {
      status = k213_getSubImage8 ((uint8) j, image);
      if (status < 0)
	{
	  printf ("Network Communication Problem\r\n");
	}
      else
	{
	  printf ("8 pixel image of 8 bits from the pixel %ld ... ", j);
	  for (i = 0; i < 8; i++)
	    printf ("%d ", image[i]);
	  printf ("\r\n\r\n\r\n\r\n");
	}
    }
  free (image);
}

/* Function 5 */
/* >>>>>>>>>> */

void
function_5 ()
{
  int32 status;
  uint8 *image;
  uint32 i, j;

  image = (uint8 *) malloc (16 * sizeof (uint8));
  if (image == 0)
    {
      printf ("Memory problem\r\n");
      return;
    }

  for (j = 0; j < 64; j++)
    {
      status = k213_getSubImage16 ((uint8) j, image);
      if (status < 0)
	{
	  printf ("Network Communication Problem\r\n");
	}
      else
	{
	  printf ("16 pixel image of 8 bits from the pixel %ld ... ", j);
	  for (i = 0; i < 16; i++)
	    printf ("%d ", image[i]);
	  printf ("\r\n\r\n\r\n\r\n");
	}
    }
  free (image);
}

/* Function 6 */
/* >>>>>>>>>> */

void
function_6 ()
{
  int32 status;
  uint8 *image;
  uint32 i;

  image = (uint8 *) malloc (32 * sizeof (uint8));
  if (image == 0)
    {
      printf ("Memory problem\r\n");
      return;
    }

  status = k213_getSubScan2Image (image);
  if (status < 0)
    {
      printf ("Network Communication Problem\r\n");
    }
  else
    {
      printf ("32 pixel image of 8 bits (1 each 2) ... ");
      for (i = 0; i < 32; i++)
	printf ("%d ", image[i]);
      printf ("\r\n\r\n\r\n\r\n");
    }
  free (image);
}

/* Function 7 */
/* >>>>>>>>>> */

void
function_7 ()
{
  int32 status;
  uint8 *image;
  uint32 i;

  image = (uint8 *) malloc (16 * sizeof (uint8));
  if (image == 0)
    {
      printf ("Memory problem\r\n");
      return;
    }

  status = k213_getSubScan4Image (image);
  if (status < 0)
    {
      printf ("Network Communication Problem\r\n");
    }
  else
    {
      printf ("16 pixel image of 8 bits (1 each 4) ... ");
      for (i = 0; i < 16; i++)
	printf ("%d ", image[i]);
      printf ("\r\n\r\n\r\n\r\n");
    }
  free (image);
}

/* Function 8 */
/* >>>>>>>>>> */

void
function_8 ()
{
  int32 status;
  uint32 i;

  for (i = 0; i < 7; i++)
    {
      tim_suspend_task (10000);
      status = k213_setScanPeriod (i);
      if (status < 0)
	{
	  printf ("Network Communication Problem\r\n");
	}
    }
}

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */

void
process_0 ()
{

  for (;;)
    {
      tim_suspend_task (500);
      var_change_led (0);
    }
}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * Launch a function.
 *
 */

void
process_1 ()
{

  for (;;)
    {
      tim_suspend_task (10);
      function_8 ();
    }
}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

int
main (void)
{
  int32 status;

  /*
     --------------------------|------------------------------------------------|
   */

  static char prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
  static char prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";

  var_reset ();
  k213_reset ();

  status = install_task (prName_0, 800, process_0);
  if (status == -1)
    exit (0);
  vIDProcess[0] = (uint32) status;

  status = install_task (prName_1, 800, process_1);
  if (status == -1)
    exit (0);
  vIDProcess[1] = (uint32) status;

  return 0;
}
