/*
; BIOS and COM system call and definitions.
; =========================================

;------------------------------------------------------------------------
; Author:	Franzi Edo.	The 03/04/98
; Modifs:	Olivier Carmona The 05/08/99
;
; Project:	KHEPERA
; Goal:		BIOS and COM system call and definitions.
;
;------------------------------------------------------------------------
*/

#ifndef	_KOS_H
#define	_KOS_H

#include <sys/ktypes.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* Khepriv */
/* ======= */

#define	return_unlock(x)	return(tim_unlock(), x)

/* Private BIOS system calls */
/* ========================= */

extern  void    tim_run_kernel(void);

#define	aBigPtr			0x400		/* location of a big ptr on some useful ptr */

/* Important structures */
/* ==================== */

typedef	struct	irSensor		IRSENSOR;
typedef	struct	procDesc		PROCDESC;
typedef	struct	process			PROCESS;
typedef	struct	turret			TURRET;
typedef	struct	headerModule	HEADERMODULE;
typedef	struct	bigPtrTable		BIGPTRTABLE;

/* Structure of the IR sensor value */
/* -------------------------------- */

struct	irSensor
	{
	#define		SENSORNB	16

	uint16		oProximitySensor[SENSORNB];		/* value of the proximity sensors     */
	uint16		oAmbientLightSensor[SENSORNB];	/* value of the ambient light sensors */
	};

/* Structure of the process descriptor of the microkernel */
/* ------------------------------------------------------ */

struct	procDesc
	{
	uint32		oIDProc;		/* process ID					 */
	uint8		oStaProc;		/* process state				 */
	#define		bPres	 	0	/* process present				 */
	#define		bLock		1	/* process lock					 */

	uint8		oReserve;		/* reserve      				 */
	uint32		oMskEvent;		/* event mask					 */
	uint32		oMskSync;		/* synchro mask					 */
	PROCDESC	*oPtrBelong;	/* ptr on the belong descriptor	 */
	PROCDESC	*oPtrBack;		/* ptr on the back descriptor	 */
	PROCDESC	*oPtrForward;	/* ptr on the forward descriptor */
	char		*oPtrTextID;	/* ptr on the descriptor ID text */
	uint32		oTStopProc;		/* a wait time					 */
	uint32		*oPtrStkPro;	/* process stack				 */
	};

/* Structure of the process descriptor (pointed by (oDescProc,Ax)) */
/* --------------------------------------------------------------- */

struct	process
	{
	uint32	*oPtrProcCode;		/* ptr on the process code			 */
	uint32	*oPtrProcStack;		/* ptr on the process stack			 */
	char	*oPtrProcText;		/* ptr on the process text ID string */
	};

/* Structure of the turret table */
/* ----------------------------- */

struct	turret
	{
	uint16	oIDEnt;			/* turret number and haeder		*/
	uint8	oLnMSGGlob;		/* size of the order (complete)	*/
	uint8	oLnMSGUtil;		/* size of the order (usuable)	*/
	uint8	oLnRep;			/* size of the answer			*/
	uint8	oQuitTour;		/* acknowledge					*/
	};

/* Structure of the header module */
/* ------------------------------ */

struct	headerModule
	{
	uint32	oBRACode;			/* jump to the module												  */
	uint32	oIDModule;			/* ID of the module T T n n											  */
	#define	kIDFunction	'FU'	/*					F U x x ---> stand alone executable function code */
	#define	kIDProtocol	'PR'	/*					P R x x ---> SerCom protocol function code		  */
	#define	kIDTool		'TO'	/*					T O x x ---> SerCom tool function code			  */
	#define	kIDBIOS		'BI'	/*					B I x x ---> BIOS								  */
	#define	kIDStartUp	'ST'	/*					S T x x ---> Start-up code						  */
	#define	kIDLib		'LI'	/*					L I x x ---> LIB								  */
	#define	kIDDebug	'DG'	/*					D G x x ---> Debug								  */
	#define	kIDProcess	'TA'	/*					T A x x ---> Proc								  */

	uint32	*oPtrCODE;			/* ptr on the CODE segment								 			  */
	uint32	oLnCODE;			/* lenght of the CODE segment								  		  */
	uint32	*oPtrDATA;			/* ptr on the DATA segment								  			  */
	uint32	oLnDATA;			/* lenght of the DATA segment								  		  */
	uint32	*oPtrStrAppl;		/* ptr on the string								  				  */
	uint32	oRevision;			/* revision								 					 		  */
	uint32	*oPtrStrHelp;		/* ptr on the help string								  			  */
	uint32	oFlType;			/* type of module								 					  */
	#define	bShow			0	/* visible in help								 					  */
	#define	bExecSerCom		1	/* executable from SerCom								 			  */
	#define	bExecRun		2	/* executable from run								 				  */
	#define	bConfidential	3	/* no comment								 					  	  */

	PROCESS	*oDescProc;			/* ptr on the process descriptor								  	  */
	uint32	oRes2;				/* reserve 2								 					  	  */
	uint32	oRes1;				/* reserve 1								 					  	  */
	};

/* Structure of the main big ptr table */
/* ----------------------------------- */

struct	bigPtrTable
	{
	HEADERMODULE	*oPtrFrstMod;		/* ptr on a first module			*/
	TURRET			*oPtrTurOTable;		/* ptr on a turret order table		*/
	uint32			*oPtrIOChan;		/* ptr on a I/O channel device		*/
	uint32			*oPtrIDComm;		/* ptr on a ID communication turret	*/
	uint32			*oPtrStUPSys;		/* ptr on a flash start-up system	*/
	uint32			*oPtrStUPUsr;		/* ptr on a flash start-up user		*/
	uint32			*oPtrFrstFreeMe;	/* ptr on a first free memory		*/
	uint32			*oPtrDownLCode;		/* ptr on a downloaded code			*/
	};

/* BIOS system calls */
/* ================= */

/* Special macros & calls (not original BIOS functions) to control the kernel */
/* -------------------------------------------------------------------------- */

#define	RESERVE_SER		while (ser_reserve_channel() == -1) tim_switch_fast();
#define	RELEASE_SER		ser_release_channel();

#define	RESERVE_MSG(channelNb)	while (msg_reserve_channel(channelNb) == -1) tim_switch_fast();
#define	RELEASE_MSG(channelNb)	msg_release_channel(channelNb);

#define	RESERVE_COM		while (com_reserve_channel() == -1) tim_switch_fast();
#define	RELEASE_COM		com_release_channel();

#define	RESERVE_CTR		while (ctr_reserve_channel() == -1) tim_switch_fast();
#define	RELEASE_CTR		ctr_release_channel();

#define	RESERVE_MMA0	while (mma_reserve_channel_0() == -1) tim_switch_fast();
#define	RELEASE_MMA0	mma_release_channel_0();

#define	RESERVE_MMA1	while (mma_reserve_channel_1() == -1) tim_switch_fast();
#define	RELEASE_MMA1	mma_release_channel_1();

#define	RESERVE_MMA2	while (mma_reserve_channel_2() == -1) tim_switch_fast();
#define	RELEASE_MMA2	mma_release_channel_2();

#define	DEBUGGER		asm("	trap	#7");

extern	int32	install_task(char *textId, uint32 stackLength, void (*procedure)());
extern	int32	kill_task(uint32 id);

/* BIOS 0..15 */
/* ---------- */

extern	void		bios_reset(void);
extern	char		*bios_get_ident(void);
extern	uint32		bios_get_rev(void);
extern	uint32		bios_get_system(void);
extern	void		bios_restart_system(void);

/* Old system calls (version < 5) */

extern	uint32		bios_get_robot(void);
extern	void		bios_restart_robot(void);

/* TIM 16..47 */
/* ---------- */

extern	void		tim_reset(void);
extern	int32		tim_new_inst_task(char *textId, uint32 *stack, void (*procedure)());
extern	int32		tim_remove_inst_task(uint32 id);
extern	void		tim_suspend_task(uint32 time);
extern	void		tim_generate_event(void);
extern	void		tim_wait_event(uint32 taskMask);
extern	uint32		tim_get_id(void);
extern	uint32		tim_get_ticcount(void);
extern	void		tim_switch_fast(void);
extern	void		tim_lock(void);
extern	void		tim_unlock(void);
extern	int32		tim_define_association(char *reference, uint32 *general);
extern	int32		tim_find_association(char *reference);
extern	int32		tim_remove_association(char *reference);
extern	void		tim_wait_sync(uint32 syncMask);
extern	PROCDESC	*tim_get_task_des_ptr(void);

/* Old system calls (version < 5) */

extern	int32		tim_new_task(void (*procedure)());
extern	int32		tim_remove_task(uint32 id);
extern	void		tim_start_kernel(void);

/* MOT 48..63 */
/* ---------- */

extern	void		mot_reset(void);
extern	int32		mot_config_speed_1m(uint32 motorNb, uint32 kp, uint32 ki, uint32 kd);
extern	int32		mot_new_speed_1m(uint32 motorNb, int32 speed);
extern	int32		mot_get_position(uint32 motorNb);
extern	int32		mot_get_speed(uint32 motorNb);
extern	int32		mot_put_sensors_1m(uint32 motorNb, int32 position);
extern	void		mot_stop(void);
extern	int32		mot_new_position_1m(uint32 motorNb, int32 position);
extern	int32		mot_new_pwm_1m(uint32 motorNb, int32 pwm);
extern	void		mot_new_speed_2m(int32 speed1, int32 speed0);
extern	int32		mot_config_position_1m(uint32 motorNb, uint32 kp, uint32 ki, uint32 kd);
extern	void		mot_put_sensors_2m(int32 position1, int32 position0);
extern	void		mot_new_position_2m(int32 position1, int32 position0);
extern	int32		mot_config_profil_1m(uint32 motorNb, uint32 maxSpeed, uint32 maxAcceleration);
extern	int32		mot_get_status(uint32 motorNb);
extern	void		mot_new_pwm_2m(int32 pwm1, int32 pwm0);

/* Old system calls (version < 5) */

extern	int32		mot_config_pid(uint32 motorNb, uint32 kp, uint32 ki, uint32 kd);
extern	int32		mot_new_speed_pid(uint32 motorNb, int32 speed);
extern	int32		mot_put_position(uint32 motorNb, int32 position);

/* SENS 64..79 */
/* ----------- */

extern	void		sens_reset(void);
extern	int32		sens_get_reflected_value(uint32 sensorNb);
extern	int32		sens_get_ambient_value(uint32 sensorNb);
extern	IRSENSOR	*sens_get_pointer(void);
extern	int32		sens_get_ana_value(uint32 inputNb);

/* MSG 80..95 */
/* ---------- */

extern	void		msg_reset(void);
extern	int32		msg_reserve_channel(uint32 numChannel);
extern	void		msg_release_channel(uint32 numChannel);
extern	int32		msg_send_message(uint8 *mesgS, uint32 sizeS);
extern	int32		msg_receive_message(uint8 *mesgR, uint32 sizeR);
extern	int32		msg_snd_rec_message(uint8 *mesgS, uint32 sizeS, uint8 *mesgR, uint32 sizeR, uint32 rep);

/* VAR 96..111 */
/* ----------- */

extern	void		var_reset(void);
extern	uint32		var_get_jumper(void);
extern	int32		var_on_led(uint32 ledNb);
extern	int32		var_off_led(uint32 ledNb);
extern	int32		var_change_led(uint32 ledNb);
extern	void		var_set_irq_vector(void(*procedure)());
extern	void		var_enable_irq(void);
extern	void		var_disable_irq(void);
extern	int32		var_set_exception(void(*procedure)(), uint32 exceptionNb);
extern	int32		var_cpu_speed(uint32 CPUSpeedValue);
extern	uint32		var_get_extension(uint32 address);
extern	void		var_put_extension(uint32 address, uint32 binaryValue);

/* Old system calls (version < 5) */

extern	void		var_set_irq6_vector(void(*procedure)());
extern	void		var_enable_irq6(void);
extern	void		var_disable_irq6(void);

/* SER 112..127 */
/* ------------ */

extern	void		ser_reset(void);
extern	int32		ser_reserve_channel(void);
extern	void		ser_release_channel(void);
extern	int32		ser_config(uint32 baudrate);
extern	int32		ser_send_buffer(uint8 *buffer, uint32 size);
extern	int32		ser_receive_byte(void);
extern	int32		ser_tx_status(void);
extern	int32		ser_rx_status(void);

/* STR 128..143 */
/* ------------ */

extern	void		str_reset(void);
extern	int32		str_cnvt_dascii_bin32(uint32 *binary, char *ascii, uint32 paramNb);
extern	int32		str_cnvt_dascii_bin8(uint8 *binary, char *ascii, uint32 paramNb);
extern	void		str_cnvt_bin32_dascii(char *ascii, uint32 *binary, uint32 paramNb);
extern	void		str_cnvt_bin8_dascii(char *ascii, uint8 *binary, uint32 paramNb);
extern	uint32		str_get_size_ascii(char *ascii);
extern	int32		str_skip_param_protocol(char *ascii, uint32 paramNb);
extern	char		*str_cnvt_vbin_dascii(char *ascii, uint32 binaryValue);
extern	int32		str_skip_param(char *ascii, uint32 paramNb);
extern	char		*str_cnvt_vbin_hascii(char *ascii, uint32 binaryValue);
extern	int32		str_cnvt_hascii_bin32(uint32 *binary, char *ascii, uint32 paramNb);
extern	int32		str_cnvt_hascii_bin8(uint8 *binary, char *ascii, uint32 paramNb);
extern	void		str_cnvt_bin32_hascii(char *ascii, uint32 *binary, uint32 paramNb);
extern	void		str_cnvt_bin8_hascii(char *ascii, uint8 *binary, uint32 paramNb);

/* Old system calls (version < 5) */

extern	int32		str_cnvt_ascii_bin32(uint32 *binary, char *ascii, uint32 paramNb);
extern	int32		str_cnvt_ascii_bin8(uint8 *binary, char *ascii, uint32 paramNb);
extern	void		str_cnvt_bin32_ascii(char *ascii, uint32 *binary, uint32 paramNb);
extern	void		str_cnvt_bin8_ascii(char *ascii, uint8 *binary, uint32 paramNb);
extern	char	   	*str_cnvt_vbin_ascii(char *ascii, uint32 binaryValue);
  
/* CTR 144..175 */
/* ------------ */

extern	void		ctr_reset(void);
extern	int32		ctr_get_ana_value(uint32 inputNb);
extern	int32		ctr_reserve_channel(void);
extern	void		ctr_release_channel(void);
extern	uint32		ctr_get_eeprom(uint32 address);
extern	void		ctr_put_eeprom(uint32 address, uint32 binaryValue);
extern 	uint32		ctr_read_battery_capacity(void);
extern 	uint32		ctr_write_battery_capacity(uint32 newCapacity);
extern 	uint32		ctr_check_on_charger(void);
extern 	void		ctr_begin_new_charge(uint32 chargeCurrent);
extern	void		ctr_stop_actual_charge(void);

/* UIO 176..207 */
/* ------------ */

extern	void		uio_reset(void);
extern	int32		uio_on_out(uint32 outNb);
extern	int32		uio_off_out(uint32 outNb);
extern	int32		uio_change_out(uint32 outNb);
extern	uint32		uio_get_inputs(void);
extern	void		uio_set_msk_irq_inputs(uint32 intMask);
extern	void		uio_ack_irq_inputs(uint32 intMask);
extern	uint32		uio_get_status_irq_inputs(void);
extern	int32		uio_get_input_state(uint32 inNb);

/* MMA 272..303 */
/* ------------ */

extern	void		mma_reset(void);
extern	int32		mma_reserve_channel_0(void);
extern	int32		mma_reserve_channel_1(void);
extern	int32		mma_reserve_channel_2(void);
extern	void		mma_release_channel_0(void);
extern	void		mma_release_channel_1(void);
extern	void		mma_release_channel_2(void);
extern	int32		mma_send_buffer_0(uint8 *buffer, uint32 size);
extern	int32		mma_send_buffer_1(uint8 *buffer, uint32 size);
extern	int32		mma_send_buffer_2(uint8 *buffer, uint32 size);
extern	int32		mma_receive_byte_0(void);
extern	int32		mma_receive_byte_1(void);
extern	int32		mma_receive_byte_2(void);
extern	int32		mma_tx_status_0(void);
extern	int32		mma_tx_status_1(void);
extern	int32		mma_tx_status_2(void);
extern	int32		mma_rx_status_0(void);
extern	int32		mma_rx_status_1(void);
extern	int32		mma_rx_status_2(void);

/* COM call system */
/* =============== */

/* COM 0..15 */
/* --------- */

extern	void		com_reset(void);
extern	int32		com_reserve_channel();
extern	void		com_release_channel();
extern	int32		com_send_buffer(uint8 *buffer, uint32 size);
extern	int32		com_receive_byte(void);
extern	int32		com_get_status_channel(void);

#ifdef  __cplusplus
}
#endif

#endif
